<template>
  <div>
    <input
      ref="excel-upload-input"
      class="excel-upload-input"
      type="file"
      accept=".xlsx, .xls"
      @change="handleClick"
    >
    <el-button
      :loading="loading"
      style="margin-left:16px;"
      :disabled="disabled"
      type="primary"
      icon="el-icon-upload2"
      waves
      @click="handleUpload"
    >
      文件上传
    </el-button>
  </div>
</template>

<script>
import XLSX from 'xlsx'
import waves from '@/directive/waves'

export default {
  directives: { waves },
  props: {
    disabled: Boolean,
    beforeUpload: Function, // eslint-disable-line
    onSuccess: Function// eslint-disable-line
  },
  data() {
    return {
      loading: false,
      excelData: {
        header: null,
        results: null,
        sheetName: null
      }
    }
  },
  methods: {
    generateData({ header, results }) {
      this.excelData.header = header // 用做el-table表头
      this.excelData.results = results // 用做el-table表体
      this.onSuccess && this.onSuccess(this.excelData)

      // 数组对象更换键名
      // const aaa = JSON.parse(JSON.stringify(results))
      // const bb = aaa.map(({
      //   序号, 姓名
      // }) => ({
      //   index: 序号,
      //   name: 姓名
      // }))
      // console.log(bb, 321)
    },
    handleUpload() {
      this.$refs['excel-upload-input'].click()
    },
    handleClick(e) {
      const files = e.target.files
      const rawFile = files[0] // only use files[0]
      if (!rawFile) return
      this.upload(rawFile)
    },
    upload(rawFile) {
      this.$refs['excel-upload-input'].value = null // fix can't select the same excel
      if (!this.beforeUpload) {
        this.readerData(rawFile)
        return
      }
      const before = this.beforeUpload(rawFile)
      if (before) { // 执行了
        this.readerData(rawFile)
      }
    },
    readerData(rawFile) {
      this.loading = true
      return new Promise((resolve, reject) => {
        const reader = new FileReader()
        reader.onload = e => {
          const data = e.target.result
          const workbook = XLSX.read(data, { type: 'array', cellDates: true })
          let worksheet = []
          const header = []
          const results = []
          workbook.SheetNames.forEach((item, index) => {
            worksheet = workbook.Sheets[item]
            header.push(this.getHeaderRow(worksheet))
            results.push(XLSX.utils.sheet_to_json(worksheet, { defval: null, raw: false }))
          })
          this.excelData.sheetName = workbook.SheetNames
          // console.log(header)
          // console.log(results)
          // const firstSheetName = workbook.SheetNames[0]
          // const worksheet = workbook.Sheets[firstSheetName]
          // const header = this.getHeaderRow(worksheet)
          // const results = XLSX.utils.sheet_to_json(worksheet)
          this.generateData({ header, results })
          this.loading = false
          resolve()
        }
        reader.readAsArrayBuffer(rawFile)
      })
    },
    getHeaderRow(sheet) {
      const headers = []
      const range = XLSX.utils.decode_range(sheet['!ref'])
      let C
      const R = range.s.r
      /* start in the first row */
      for (C = range.s.c; C <= range.e.c; ++C) { /* walk every column in the range */
        const cell = sheet[XLSX.utils.encode_cell({ c: C, r: R })]
        /* find the cell in the first row */
        let hdr = 'UNKNOWN ' + C // <-- replace with your desired default
        if (cell && cell.t) hdr = XLSX.utils.format_cell(cell)
        headers.push(hdr)
      }
      return headers
    },
    isExcel(file) {
      return /\.(xlsx|xls|csv)$/.test(file.name)
    }
  }
}
</script>

<style scoped>
.excel-upload-input {
  display: none;
  z-index: -9999;
}

.drop {
  border: 2px dashed #bbb;
  width: 600px;
  height: 160px;
  line-height: 160px;
  margin: 0 auto;
  font-size: 24px;
  border-radius: 5px;
  text-align: center;
  color: #bbb;
  position: relative;
}

</style>

<style>

/*.has-gutter{*/
/*  color:  #34bfa3 !important;*/
/*}*/

/*.is-leaf{*/
/*  background-color: #34bfa3 !important;*/
/*}*/
</style>
